@@ -208,7 +208,7 @@ class FleurContext(object):
 
     def onClose_x_fleur_section_XC(self, backend, gIndex, section):
         xc_index = section["x_fleur_exch_pot"]
-        logging.error("winsectxc: %s -> %s", section, xc_index)
+        logging.info("winsectxc: %s -> %s", section, xc_index)
         if not xc_index:
             xc_index = ["pbe"]
         xc_map_legend = {
@@ -322,7 +322,7 @@ mainFileDescription = SM(
                     SM(name = 'header',
                     startReStr = r"\s* This output is generated by\s*(?P<x_fleur_version>[\w*.]+)\s*\w*\*\s\*",
                     sections=["x_fleur_header"],
-                    fixedStartValues={'program_name': 'fleur', 'program_basis_set_type': 'FLAPW' }
+                    fixedStartValues={'program_name': 'fleur', 'program_basis_set_type': '(L)APW+lo' }
                    ),
 
                     SM(name = 'systemName',
