@@ -1,11 +1,11 @@
 # Copyright 2015-2018 Ask Hjorth Larsen, Fawzi Mohamed, Ankit Kariryaa
-# 
+#
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
 #   You may obtain a copy of the License at
-# 
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 #   Unless required by applicable law or agreed to in writing, software
 #   distributed under the License is distributed on an "AS IS" BASIS,
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -63,7 +63,6 @@ def read_eigenvalues_file(fd):
             eigs[-1].setdefault(spin, []).append(float(eig))
             occs[-1].setdefault(spin, []).append(float(occ))
 
-
     nkpts = len(kpts)
     nspins = len(eigs[0])
     nbands = len(eigs[0][spin])
@@ -180,6 +179,7 @@ def block2list(namespace, lines, header=None):
         block.append(tokens)
     return name, block
 
+
 class OctNamespace:
     def __init__(self):
         self.names = {}
@@ -310,15 +310,33 @@ def read_static_info_kpoints(fd):
     return dict(ibz_k_points=ibz_k_points, k_point_weights=k_point_weights)
 
 
-def read_static_info_eigenvalues(fd, energy_unit):
+def read_static_info_eigenvalues_efermi(fd, energy_unit):
+    '''
+    Parse eigenvalues and Fermi Energy from `static/info`
+    fd: file descriptor
+    energy_unit: string
+    Returns: dictionary
+    '''
+
+    # we had to allow this function to handle both
+    # eigenvalues and Fermi energies because the later
+    # comes in the output file immediately after the last
+    # eigenvalue, hence the "fermi line" is consumed by the
+    # line checkers that look for eigenvalues.
 
     values_sknx = {}
 
+    eFermi = None
     nbands = 0
     for line in fd:
         line = line.strip()
         if line.startswith('#'):
             continue
+        if line.startswith('Fermi'):  # tmk
+            # print(line, '##') #  OK!
+            tokens = line.split()
+            unit = {'eV': eV, 'H': Hartree}[tokens[-1]]
+            eFermi = float(tokens[-2]) * unit
         if not line[:1].isdigit():
             break
 
@@ -342,11 +360,16 @@ def read_static_info_eigenvalues(fd, energy_unit):
     eps_skn = eps_skn.transpose(1, 0, 2).copy()
     occ_skn = occ_skn.transpose(1, 0, 2).copy()
     assert eps_skn.flags.contiguous
-    return dict(nspins=nspins,
-                nkpts=nkpts,
-                nbands=nbands,
-                eigenvalues=eps_skn,
-                occupations=occ_skn)
+    # print('save to dictionary: ', nspins, nkpts, nbands, eps_skn, occ_skn, eFermi)
+    info_dict = dict(nspins=nspins,
+                     nkpts=nkpts,
+                     nbands=nbands,
+                     eigenvalues=eps_skn,
+                     occupations=occ_skn)
+    if eFermi is not None:
+        # not all Octopus' output might report the Fermi energy
+        info_dict['efermi'] = eFermi
+    return info_dict
 
 
 def read_static_info_energy(fd, energy_unit):
@@ -358,8 +381,8 @@ def read_static_info_energy(fd, energy_unit):
 
 
 def read_static_info(fd):
+    # fd: file descriptor
     results = {}
-
     def get_energy_unit(line):  # Convert "title [unit]": ---> unit
         return {'[eV]': eV, '[H]': Hartree}[line.split()[1].rstrip(':')]
 
@@ -368,7 +391,7 @@ def read_static_info(fd):
             results.update(read_static_info_kpoints(fd))
         elif line.startswith('Eigenvalues ['):
             unit = get_energy_unit(line)
-            results.update(read_static_info_eigenvalues(fd, unit))
+            results.update(read_static_info_eigenvalues_efermi(fd, unit))
         elif line.startswith('Energy ['):
             unit = get_energy_unit(line)
             results.update(read_static_info_energy(fd, unit))
@@ -409,11 +432,6 @@ def read_static_info(fd):
                 tokens = line.split()[-3:]
                 forces.append([float(f) for f in tokens])
             results['forces'] = np.array(forces) * forceunit
-        elif line.startswith('Fermi'):
-            tokens = line.split()
-            unit = {'eV': eV, 'H': Hartree}[tokens[-1]]
-            eFermi = float(tokens[-2]) * unit
-            results['efermi'] = eFermi
 
     if 'ibz_k_points' not in results:
         results['ibz_k_points'] = np.zeros((1, 3))
@@ -430,7 +448,8 @@ def read_static_info(fd):
                 break
         eFermi = all_energies[arg]
         results['efermi'] = eFermi
-
+        # print('eFermi estimate', eFermi) # this produces bad estimate
+        # ----
     return results
 
 
@@ -466,6 +485,9 @@ class Octopus(FileIOCalculator):
     def get_fermi_level(self):
         return self.results['efermi']
 
+    def get_fermi_energy(self):
+        return self.results['efermi']
+
     def get_dipole_moment(self, atoms=None):
         if 'dipole' not in self.results:
             msg = ('Dipole moment not calculated.\n'
@@ -598,7 +620,7 @@ class Octopus(FileIOCalculator):
             return False
         elif sc == 'spin_polarized' or sc == 'polarized':
             return True
-        #else:
+        # else:
         #    raise NotImplementedError('SpinComponents keyword %s' % sc)
 
     def get_ibz_k_points(self):
@@ -668,7 +690,7 @@ class Octopus(FileIOCalculator):
         fd = open(inp_path)
         kwargs = parse_input_file(fd)
 
-        #self.atoms, kwargs = kwargs2atoms(kwargs)
+        # self.atoms, kwargs = kwargs2atoms(kwargs)
         self.kwargs.update(kwargs)
 
         fd.close()
