@@ -14,7 +14,7 @@
 
 ''' All generic entry metadata and related classes. '''
 
-from typing import Dict, Any
+from typing import Any
 from cachetools import cached, TTLCache
 from elasticsearch_dsl import Keyword, Text, analyzer, tokenizer
 import ase.data
@@ -23,9 +23,11 @@ from nomad import metainfo, config
 from nomad.metainfo.search_extension import Search
 from nomad.metainfo.elastic_extension import ElasticDocument
 from nomad.metainfo.mongoengine_extension import Mongo, MongoDocument
+from nomad.datamodel.metainfo.public import fast_access
 
 from .dft import DFTMetadata
 from .ems import EMSMetadata
+from .qcms import QCMSMetadata
 
 # This is usually defined automatically when the first metainfo definition is evaluated, but
 # due to the next imports requireing the m_package already, this would be too late.
@@ -33,7 +35,8 @@ m_package = metainfo.Package()
 
 from .encyclopedia import EncyclopediaMetadata  # noqa
 from .metainfo.public import section_run, Workflow  # noqa
-from .metainfo.general_experimental import section_experiment  # noqa
+from .metainfo.common_experimental import Experiment  # noqa
+from .metainfo.general_qcms import QuantumCMS  # noqa
 
 
 def _only_atoms(atoms):
@@ -47,7 +50,25 @@ path_analyzer = analyzer(
     tokenizer=tokenizer('path_tokenizer', 'pattern', pattern='/'))
 
 
-class User(metainfo.MSection):
+class Author(metainfo.MSection):
+    ''' A person that is author of data in NOMAD or references by NOMAD. '''
+    name = metainfo.Quantity(
+        type=str,
+        derived=lambda user: ('%s %s' % (user.first_name, user.last_name)).strip(),
+        a_search=Search(mapping=Text(fields={'keyword': Keyword()})))
+
+    first_name = metainfo.Quantity(type=metainfo.Capitalized)
+    last_name = metainfo.Quantity(type=metainfo.Capitalized)
+    email = metainfo.Quantity(
+        type=str,
+        a_elastic=dict(mapping=Keyword),  # TODO remove?
+        a_search=Search())
+
+    affiliation = metainfo.Quantity(type=str)
+    affiliation_address = metainfo.Quantity(type=str)
+
+
+class User(Author):
     ''' A NOMAD user.
 
     Typically a NOMAD user has a NOMAD account. The user related data is managed by
@@ -70,23 +91,14 @@ class User(metainfo.MSection):
         type=str,
         a_search=Search())
 
-    name = metainfo.Quantity(
-        type=str,
-        derived=lambda user: ('%s %s' % (user.first_name, user.last_name)).strip(),
-        a_search=Search(mapping=Text(fields={'keyword': Keyword()})))
-
-    first_name = metainfo.Quantity(type=str)
-    last_name = metainfo.Quantity(type=str)
-    email = metainfo.Quantity(
-        type=str,
-        a_elastic=dict(mapping=Keyword),  # TODO remove?
-        a_search=Search())
-
     username = metainfo.Quantity(type=str)
-    affiliation = metainfo.Quantity(type=str)
-    affiliation_address = metainfo.Quantity(type=str)
+
     created = metainfo.Quantity(type=metainfo.Datetime)
-    repo_user_id = metainfo.Quantity(type=str)
+
+    repo_user_id = metainfo.Quantity(
+        type=str,
+        description='Optional, legacy user id from the old NOMAD CoE repository.')
+
     is_admin = metainfo.Quantity(
         type=bool, derived=lambda user: user.user_id == config.services.admin_user_id)
 
@@ -96,16 +108,6 @@ class User(metainfo.MSection):
         from nomad import infrastructure
         return infrastructure.keycloak.get_user(*args, **kwargs)  # type: ignore
 
-    @staticmethod
-    @cached(cache=TTLCache(maxsize=1, ttl=24 * 3600))
-    def repo_users() -> Dict[str, 'User']:
-        from nomad import infrastructure
-        return {
-            str(user.repo_user_id): user
-            for user in infrastructure.keycloak.search_user()
-            if user.repo_user_id is not None
-        }
-
 
 class UserReference(metainfo.Reference):
     '''
@@ -117,7 +119,7 @@ class UserReference(metainfo.Reference):
         super().__init__(User.m_def)
 
     def resolve(self, proxy: metainfo.MProxy) -> metainfo.MSection:
-        return User.get(user_id=proxy.m_proxy_url)
+        return User.get(user_id=proxy.m_proxy_value)
 
     def serialize(self, section: metainfo.MSection, quantity_def: metainfo.Quantity, value: Any) -> Any:
         return value.user_id
@@ -126,6 +128,37 @@ class UserReference(metainfo.Reference):
 user_reference = UserReference()
 
 
+class AuthorReference(metainfo.Reference):
+    '''
+    Special metainfo reference type that allows to use either user_ids or direct author
+    information as values. It automatically resolves user_ids to User objects and author
+    data into Author objects.
+    '''
+
+    def __init__(self):
+        super().__init__(Author.m_def)
+
+    def resolve(self, proxy: metainfo.MProxy) -> metainfo.MSection:
+        proxy_value = proxy.m_proxy_value
+        if isinstance(proxy_value, str):
+            return User.get(user_id=proxy.m_proxy_value)
+        elif isinstance(proxy_value, dict):
+            return Author.m_from_dict(proxy_value)
+        else:
+            raise metainfo.MetainfoReferenceError()
+
+    def serialize(self, section: metainfo.MSection, quantity_def: metainfo.Quantity, value: Any) -> Any:
+        if isinstance(value, User):
+            return value.user_id
+        elif isinstance(value, Author):
+            return value.m_to_dict()
+        else:
+            raise metainfo.MetainfoReferenceError()
+
+
+author_reference = AuthorReference()
+
+
 class Dataset(metainfo.MSection):
     ''' A Dataset is attached to one or many entries to form a set of data.
 
@@ -181,11 +214,11 @@ class DatasetReference(metainfo.Reference):
         super().__init__(Dataset.m_def)
 
     def resolve(self, proxy: metainfo.MProxy) -> metainfo.MSection:
-        return Dataset.m_def.a_mongo.get(dataset_id=proxy.m_proxy_url)
+        return Dataset.m_def.a_mongo.get(dataset_id=proxy.m_proxy_value)
 
     def serialize(self, section: metainfo.MSection, quantity_def: metainfo.Quantity, value: Any) -> Any:
         if isinstance(value, metainfo.MProxy):
-            return value.m_proxy_url
+            return value.m_proxy_value
         else:
             return value.dataset_id
 
@@ -193,8 +226,13 @@ class DatasetReference(metainfo.Reference):
 dataset_reference = DatasetReference()
 
 
+class UserProvidableMetadata(metainfo.MCategory):
+    ''' NOMAD entry metadata quantities that can be determined by the user, e.g. via nomad.yaml. '''
+
+
 class EditableUserMetadata(metainfo.MCategory):
-    ''' NOMAD entry quantities that can be edited by the user after publish. '''
+    ''' NOMAD entry metadata quantities that can be edited by the user after publish. '''
+    m_def = metainfo.Category(categories=[UserProvidableMetadata])
 
 
 class MongoMetadata(metainfo.MCategory):
@@ -207,6 +245,23 @@ class DomainMetadata(metainfo.MCategory):
     pass
 
 
+def derive_origin(entry):
+    if entry.external_db is not None:
+        return str(entry.external_db)
+
+    if entry.uploader:
+        return entry.uploader.name
+
+    return None
+
+
+def derive_authors(entry):
+    uploaders = []
+    if entry.uploader is not None and entry.external_db is None:
+        uploaders = [entry.uploader]
+    return uploaders + entry.coauthors
+
+
 class EntryMetadata(metainfo.MSection):
     '''
     Attributes:
@@ -240,30 +295,30 @@ class EntryMetadata(metainfo.MSection):
         upload_time: The time that this entry was uploaded
         datasets: Ids of all datasets that this entry appears in
     '''
-    m_def = metainfo.Section(a_elastic=ElasticDocument(
-        index_name=config.elastic.index_name, id=lambda x: x.calc_id))
+    m_def = metainfo.Section(
+        a_elastic=ElasticDocument(index_name=config.elastic.index_name, id=lambda x: x.calc_id))
 
     upload_id = metainfo.Quantity(
         type=str,
-        description='A random UUID that uniquely identifies the upload of the entry.',
+        description='The persistent and globally unique identifier for the upload of the entry',
         a_search=Search(
             many_or='append', group='uploads_grouped', metric_name='uploads', metric='cardinality'))
 
     calc_id = metainfo.Quantity(
         type=str,
-        description='A unique ID based on the upload id and entry\'s mainfile.',
+        description='A persistent and globally unique identifier for the entry',
         a_search=Search(many_or='append'))
 
     calc_hash = metainfo.Quantity(
         type=str,
-        description='A raw file content based checksum/hash.',
+        description='A raw file content based checksum/hash',
         categories=[MongoMetadata],
         a_search=Search(
             many_or='append', metric_name='unique_entries', metric='cardinality'))
 
     mainfile = metainfo.Quantity(
         type=str,
-        description='The upload relative mainfile path.',
+        description='The path to the mainfile from the root directory of the uploaded files',
         a_search=[
             Search(
                 description='Search within the mainfile path.',
@@ -275,7 +330,11 @@ class EntryMetadata(metainfo.MSection):
 
     files = metainfo.Quantity(
         type=str, shape=['0..*'],
-        description='The entries raw file paths relative to its upload.',
+        description='''
+        The paths to the files within the upload that belong to this entry.
+        All files within the same directory as the entry's mainfile are considered the
+        auxiliary files that belong to the entry.
+        ''',
         a_search=[
             Search(
                 description='Search within the paths.', name='path',
@@ -288,20 +347,25 @@ class EntryMetadata(metainfo.MSection):
 
     pid = metainfo.Quantity(
         type=str,
-        description='The unique, sequentially enumerated, integer persistent identifier',
+        description='''
+        The unique, sequentially enumerated, integer PID that was used in the legacy
+        NOMAD CoE. It allows to resolve URLs of the old NOMAD CoE Repository.''',
         categories=[MongoMetadata],
         a_search=Search(many_or='append'))
 
     raw_id = metainfo.Quantity(
         type=str,
-        description='A raw format specific id that was acquired from the files of this entry',
-        categories=[MongoMetadata],
+        description='''
+        The code specific identifier extracted from the entrie's raw files if such an
+        identifier is supported by the underlying code
+        ''',
+        categories=[MongoMetadata, UserProvidableMetadata],
         a_search=Search(many_or='append'))
 
     domain = metainfo.Quantity(
-        type=metainfo.MEnum('dft', 'ems'),
+        type=metainfo.MEnum('dft', 'ems', 'qcms'),
         description='The material science domain',
-        categories=[MongoMetadata],
+        categories=[MongoMetadata, UserProvidableMetadata],
         a_search=Search())
 
     published = metainfo.Quantity(
@@ -318,33 +382,38 @@ class EntryMetadata(metainfo.MSection):
 
     last_processing = metainfo.Quantity(
         type=metainfo.Datetime,
-        description='The datetime of the last attempted processing.',
+        description='The datetime of the last processing',
         categories=[MongoMetadata],
         a_search=Search())
 
     nomad_version = metainfo.Quantity(
         type=str,
-        description='The NOMAD version used for the last processing attempt.',
+        description='The NOMAD version used for the last processing',
         categories=[MongoMetadata],
         a_search=Search(many_or='append'))
     nomad_commit = metainfo.Quantity(
         type=str,
-        description='The NOMAD commit used for the last processing attempt.',
+        description='The NOMAD commit used for the last processing',
         categories=[MongoMetadata],
         a_search=Search(many_or='append'))
     parser_name = metainfo.Quantity(
         type=str,
-        description='The NOMAD parser used for the last processing attempt.',
+        description='The NOMAD parser used for the last processing',
         a_search=Search(many_or='append'))
 
     comment = metainfo.Quantity(
         type=str, categories=[MongoMetadata, EditableUserMetadata],
-        description='A user provided comment.',
+        description='A user provided comment for this entry',
         a_search=Search(mapping=Text()))
 
     references = metainfo.Quantity(
         type=str, shape=['0..*'], categories=[MongoMetadata, EditableUserMetadata],
-        description='User provided references (URLs).',
+        description='User provided references (URLs) for this entry',
+        a_search=Search())
+
+    external_db = metainfo.Quantity(
+        type=metainfo.MEnum('EELSDB'), categories=[MongoMetadata, UserProvidableMetadata],
+        description='The repository or external database where the original data resides',
         a_search=Search())
 
     uploader = metainfo.Quantity(
@@ -353,40 +422,50 @@ class EntryMetadata(metainfo.MSection):
         a_flask=dict(admin_only=True, verify=User),
         a_search=[
             Search(
-                description='Search uploader with exact names.',
+                description='The full name of the authors for exact searches',
                 metric_name='uploaders', metric='cardinality',
                 many_or='append', search_field='uploader.name.keyword',
                 statistic_size=10,
                 statistic_order='_count'),
             Search(
-                name='uploader_id', search_field='uploader.user_id')
+                name='uploader_id', search_field='uploader.user_id',
+                description='The full name of the authors',)
         ])
 
+    origin = metainfo.Quantity(
+        type=str,
+        description='''
+            A short human readable description of the entries origin. Usually it is the
+            handle of an external database/repository or the name of the uploader.
+        ''',
+        derived=derive_origin,
+        a_search=Search(statistic_size=10, statistic_order='_count'))
+
     coauthors = metainfo.Quantity(
-        type=user_reference, shape=['0..*'], default=[], categories=[MongoMetadata, EditableUserMetadata],
-        description='A user provided list of co-authors.',
+        type=author_reference, shape=['0..*'], default=[], categories=[MongoMetadata, EditableUserMetadata],
+        description='A user provided list of co-authors',
         a_flask=dict(verify=User))
 
     authors = metainfo.Quantity(
-        type=user_reference, shape=['0..*'],
-        description='All authors (uploader and co-authors).',
-        derived=lambda entry: ([entry.uploader] if entry.uploader is not None else []) + entry.coauthors,
+        type=author_reference, shape=['0..*'],
+        description='All authors (uploader and co-authors)',
+        derived=derive_authors,
         a_search=Search(
-            description='Search authors with exact names.',
+            description='The full name of the authors for exact searches',
             metric='cardinality',
             many_or='append', search_field='authors.name.keyword'))
 
     shared_with = metainfo.Quantity(
         type=user_reference, shape=['0..*'], default=[], categories=[MongoMetadata, EditableUserMetadata],
-        description='A user provided list of userts to share the entry with.',
+        description='A user provided list of userts to share the entry with',
         a_flask=dict(verify=User))
 
     owners = metainfo.Quantity(
         type=user_reference, shape=['0..*'],
-        description='All owner (uploader and shared with users).',
+        description='All owner (uploader and shared with users)',
         derived=lambda entry: ([entry.uploader] if entry.uploader is not None else []) + entry.shared_with,
         a_search=Search(
-            description='Search owner with exact names.',
+            description='The full name of the owners for exact searches',
             many_or='append', search_field='owners.name.keyword'))
 
     with_embargo = metainfo.Quantity(
@@ -396,7 +475,7 @@ class EntryMetadata(metainfo.MSection):
 
     upload_time = metainfo.Quantity(
         type=metainfo.Datetime, categories=[MongoMetadata],
-        description='The datetime this entry was uploaded to nomad',
+        description='The date and time this entry was uploaded to nomad',
         a_flask=dict(admin_only=True),
         a_search=Search(order_default=True))
 
@@ -413,37 +492,39 @@ class EntryMetadata(metainfo.MSection):
         a_search=[
             Search(
                 search_field='datasets.name', many_or='append',
-                description='Search for a particular dataset by exact name.'),
+                description='A list of user curated datasets this entry belongs to for exact name search'),
             Search(
                 name='dataset_id', search_field='datasets.dataset_id', many_or='append',
                 group='datasets_grouped',
                 metric='cardinality', metric_name='datasets',
-                description='Search for a particular dataset by its id.')])
+                description='A list of user curated datasets this entry belongs to for exact name search')])
 
     external_id = metainfo.Quantity(
-        type=str, categories=[MongoMetadata],
-        description='A user provided external id.',
+        type=str, categories=[MongoMetadata, UserProvidableMetadata],
+        description='''
+        A user provided external id. Usually the id for an entry in an external database
+        where the data was imported from.''',
         a_search=Search(many_or='split'))
 
     last_edit = metainfo.Quantity(
         type=metainfo.Datetime, categories=[MongoMetadata],
-        description='The datetime the user metadata was edited last.',
+        description='The date and time the user metadata was edited last',
         a_search=Search())
 
     formula = metainfo.Quantity(
         type=str, categories=[DomainMetadata],
-        description='A (reduced) chemical formula.',
+        description='A (reduced) chemical formula',
         a_search=Search())
 
     atoms = metainfo.Quantity(
         type=str, shape=['n_atoms'], default=[], categories=[DomainMetadata],
-        description='The atom labels of all atoms of the entry\'s material.',
+        description='The atom labels of all atoms of the entry\'s material',
         a_search=Search(
             many_and='append', statistic_size=len(ase.data.chemical_symbols)))
 
     only_atoms = metainfo.Quantity(
         type=str, categories=[DomainMetadata],
-        description='The atom labels concatenated in order-number order.',
+        description='The atom labels concatenated in order-number order',
         derived=lambda entry: _only_atoms(entry.atoms),
         a_search=Search(many_and='append', derived=_only_atoms))
 
@@ -453,16 +534,16 @@ class EntryMetadata(metainfo.MSection):
         a_search=Search())
 
     ems = metainfo.SubSection(sub_section=EMSMetadata, a_search='ems')
-    dft = metainfo.SubSection(sub_section=DFTMetadata, a_search='dft')
+    dft = metainfo.SubSection(sub_section=DFTMetadata, a_search='dft', categories=[fast_access])
+    qcms = metainfo.SubSection(sub_section=QCMSMetadata, a_search='qcms')
     encyclopedia = metainfo.SubSection(sub_section=EncyclopediaMetadata, a_search='encyclopedia')
 
     def apply_user_metadata(self, metadata: dict):
         ''' Applies a user provided metadata dict to this calc. '''
         self.m_update(**metadata)
 
-    def apply_domain_metadata(self, backend):
-        """Used to apply metadata that is related to the domain.
-        """
+    def apply_domain_metadata(self, archive):
+        ''' Used to apply metadata that is related to the domain. '''
         assert self.domain is not None, 'all entries must have a domain'
         domain_sub_section_def = self.m_def.all_sub_sections.get(self.domain)
         domain_section_def = domain_sub_section_def.sub_section
@@ -473,15 +554,16 @@ class EntryMetadata(metainfo.MSection):
         if domain_section is None:
             domain_section = self.m_create(domain_section_def.section_cls)
 
-        domain_section.apply_domain_metadata(backend)
+        domain_section.apply_domain_metadata(archive)
 
 
 class EntryArchive(metainfo.MSection):
 
     section_run = metainfo.SubSection(sub_section=section_run, repeats=True)
-    section_experiment = metainfo.SubSection(sub_section=section_experiment)
-    section_workflow = metainfo.SubSection(sub_section=Workflow)
-    section_metadata = metainfo.SubSection(sub_section=EntryMetadata)
+    section_experiment = metainfo.SubSection(sub_section=Experiment)
+    section_quantum_cms = metainfo.SubSection(sub_section=QuantumCMS)
+    section_workflow = metainfo.SubSection(sub_section=Workflow, categories=[fast_access])
+    section_metadata = metainfo.SubSection(sub_section=EntryMetadata, categories=[fast_access])
 
     processing_logs = metainfo.Quantity(
         type=Any, shape=['0..*'],
